/*
 * ChessLET.java
 *
 * Created on 3. November 2007, 13:35
 */

/*
 The contents of this file are subject to the
 Mozilla Public License Version 1.1 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied.
 See the License for the specific language governing rights
 and limitations under the License.

 The Initial Developer of the Original Code is
 Georg-Johann Lay (GeorgJohann@web.de).

 Portions created by the Initial Developer are
 Copyright (C) 2007 Georg-Johann Lay. All Rights Reserved.

 Alternatively, the contents of this file may be used under the terms
 of either the GNU General Public License Version 3 or later (the "GPL"),
 or the GNU Lesser General Public License Version 3 or later (the "LGPL"),
 in which case the provisions of the GPL or the LGPL are applicable instead
 of those above.

 If you wish to allow use of your version of this file only
 under the terms of either the GPL or the LGPL, and not to allow others to
 use your version of this file under the terms of the MPL, indicate your
 decision by deleting the provisions above and replace them with the notice
 and other provisions required by the LGPL or the GPL. If you do not delete
 the provisions above, a recipient may use your version of this file under
 the terms of any one of the MPL, the GPL or the LGPL.
*/

package de.gjlay.chess;

import java.util.Vector;
import javax.microedition.midlet.MIDlet;
import javax.microedition.lcdui.*;

import de.gjlay.mobile.ui.*;
import de.gjlay.mobile.localize.*;
/**
 * The main class of the MobileChessBoard MIDlet suite.
 * @author Georg-Johann Lay
 */
public class ChessLET
        extends MIDlet
        implements Localizer, CoDec
{
    /**
     * Creates a new instance of ChessLET
     */

    static final String DEFAULT_LOCALE = "default";

    private static ChessLET instance;
    private String locale;

    public ChessLET ()
    {
        instance = this;
    }

    private void init ()
    {
        //Drawer.setMIDlet (this);
        PositionCanvas.GetInstance (this, this);
        PositionInfoForm.GetInstance (this, this);
        PlayCanvas.GetInstance (this, this);
        Selector.GetInstance (this);
        Message.GetInstance (this);
        TextForm.GetInstance (this);

        load();

        PlayCanvas.Show (true);
    }
    /**
     * Exit the MIDlet.
     */
    static void Exit()
    {
        Display.getDisplay (instance).setCurrent (null);
        instance.destroyApp (true);
        instance.notifyDestroyed ();
    }

    public void startApp ()
    {
        init ();
    }

    public void pauseApp ()
    {
    }

    public void destroyApp (boolean unconditional)
    {
    }

    ////////////////////////////////////////////////////////
    static void NotImplemented()
    {
        Message.ShowInfo (L._("T_NOT_IMPLEMENTED"));
    }
    ////////////////////////////////////////////////////////

    private Vector lalis = new Vector();

    public void addLanguageListener (LanguageListener lali)
    {
        if (!lalis.contains (lali))
            lalis.addElement (lali);
    }

    public String getLocale ()
    {
        return locale;
    }

    public void setLocale (String locale)
    {
        if (null != locale && DEFAULT_LOCALE.equals (locale))
            locale = System.getProperty ("microedition.locale");

        this.locale = locale;

        L.setLocale (locale);

        for (int i=0; i<lalis.size(); i++)
            ((LanguageListener) lalis.elementAt (i)).languageChanged (locale);

        save();
    }

 //#if CONSOLE
//#     public final static void Debug (String msg)
//#     {
//#         System.err.println (msg);
//#     }
//#endif

    public int getEncodeSize()
    {
        return encode ().length;
    }

    public byte[] encode ()
    {
        return (null == locale) ? null : locale.getBytes();
    }

    // code may be null */
    public void decode (byte[] code, Object _unused_)
    {
        locale = (null == code) ? DEFAULT_LOCALE : new String (code);
        setLocale (locale);
    }

    void load ()
    {
        decode (Lover.Load ("locale"), null);
    }

    void save ()
    {
        Lover.Save ("locale", this);
    }
}
