/*
 * History.java
 *
 * Created on 9. November 2007, 17:17
 */

/*
 The contents of this file are subject to the
 Mozilla Public License Version 1.1 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied.
 See the License for the specific language governing rights
 and limitations under the License.

 The Initial Developer of the Original Code is
 Georg-Johann Lay (GeorgJohann@web.de).

 Portions created by the Initial Developer are
 Copyright (C) 2007 Georg-Johann Lay. All Rights Reserved.

 Alternatively, the contents of this file may be used under the terms
 of either the GNU General Public License Version 3 or later (the "GPL"),
 or the GNU Lesser General Public License Version 3 or later (the "LGPL"),
 in which case the provisions of the GPL or the LGPL are applicable instead
 of those above.

 If you wish to allow use of your version of this file only
 under the terms of either the GPL or the LGPL, and not to allow others to
 use your version of this file under the terms of the MPL, indicate your
 decision by deleting the provisions above and replace them with the notice
 and other provisions required by the LGPL or the GPL. If you do not delete
 the provisions above, a recipient may use your version of this file under
 the terms of any one of the MPL, the GPL or the LGPL.
*/

package de.gjlay.chess;

/**
 *
 * @author Georg-Johann Lay
 */
class History
        extends java.util.Vector
        implements CoDec
{
    static final int DUMP_SHORT  = 0;
    static final int DUMP_STRING = 1;
    static final int DUMP_MOVE   = 2;
    
    int am = 0;
    /** Creates a new instance of History */
    History () {}

    Move getElement (int index)
    {
        return (Move) super.elementAt (index);
    }
    
    Move getActualMove()
    {
        return am <= 0 ? null : getElement (am-1);
    }
    
    void clear()
    {
        setSize (am = 0);
    }

    void apply (Board b, Move m)
    {
        setSize (am++);
        addElement (m);
        m.apply (b);
    }

    Move undo (Board b)
    {
        Move m = null;

        if (am > 0)
            (m = getElement (--am)).undo (b);

        return m;
    }

    void undoAll (Board b)
    {
        while (null != undo (b))
            ;
    }

    Move redo (Board b)
    {
        Move m = null;

        if (am < size())
            (m = getElement (am++)).apply (b);

        return m;
    }

    Move removeLast ()
    {
        Move m = null;

        if (size() > 0)
        {
            m = getElement (size()-1);
            setSize (size()-1);
            am = Math.min (am, size());
        }

        return m;
    }

    public int getEncodeSize()
    {
        return 2*(1+size());
    }

    public byte[] encode ()
    {
        byte[] code = new byte[getEncodeSize()];

        Bytes.SetShort (code, 0, (short) am);

        for (int i=0; i < size(); i++)
            Bytes.SetShort (code, 2*(1+i), getElement (i).encode());

        return code;
    }

    public void decode (byte[] code, Object info)
    {
        Board b = (Board) info;
        clear();

        for (int i=2; i < code.length; i+=2)
        {
            Move m = Move.Decode (b, Bytes.GetShort (code, i));
            apply (b,m);
            m.isCheck = b.isControlled (m.color, b.king[1-m.color]);
        }

        for (int am = Bytes.GetShort (code, 0); am < this.am; )
            undo(b);
    }

//#if CONSOLE        
//#     void dump (int mode, boolean asStringArray)
//#     {
//#         String in = asStringArray ? "            " : "        ";
//#         System.out.println (in + (asStringArray 
//#             ? "new String[] { null,"
//#             : "{   \"\","
//#         ));
//#         
//#         String line = "";
//#         
//#         for (int i=0; i < size(); i++)
//#         {
//#             Move m = getElement (i);
//#             
//#             String ms = 
//#                     (DUMP_MOVE == mode) ? "\""+m+"\""
//#                     : (DUMP_STRING == mode) ? "\""+m.toShortString()+"\""
//#                     : (DUMP_SHORT  == mode) ? ""+m.encode()
//#                     : "?";
//#             
//#             if (i % 2 == 0)
//#                 line = in + "    /* " + (1+i/2) + " */ " + ms;
//#             else
//#                 line += " " + ms;
//#             
//#             if (i < size()-1)
//#                 line += ",";
//#             
//#             if (i%2 == 1)
//#                 System.out.println (line);
//#         }
//#         
//#         if (size() % 2 == 1)
//#             System.out.println (line);
//#         
//#         if (asStringArray)    
//#             System.out.print (in+"}");
//#         else
//#             System.out.println (in+"}");
//#     }
//#endif
}
