/*
 * PositionCanvas.java
 *
 * Created on 22. November 2007, 16:29
 */

/*
 The contents of this file are subject to the
 Mozilla Public License Version 1.1 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied.
 See the License for the specific language governing rights
 and limitations under the License.

 The Initial Developer of the Original Code is
 Georg-Johann Lay (GeorgJohann@web.de).

 Portions created by the Initial Developer are
 Copyright (C) 2007 Georg-Johann Lay. All Rights Reserved.

 Alternatively, the contents of this file may be used under the terms
 of either the GNU General Public License Version 3 or later (the "GPL"),
 or the GNU Lesser General Public License Version 3 or later (the "LGPL"),
 in which case the provisions of the GPL or the LGPL are applicable instead
 of those above.

 If you wish to allow use of your version of this file only
 under the terms of either the GPL or the LGPL, and not to allow others to
 use your version of this file under the terms of the MPL, indicate your
 decision by deleting the provisions above and replace them with the notice
 and other provisions required by the LGPL or the GPL. If you do not delete
 the provisions above, a recipient may use your version of this file under
 the terms of any one of the MPL, the GPL or the LGPL.
*/

package de.gjlay.chess;

import de.gjlay.mobile.localize.*;
import de.gjlay.mobile.ui.*;
import javax.microedition.lcdui.*;
import javax.microedition.midlet.MIDlet;

/**
 *
 * @author Georg-Johann Lay
 */
class PositionCanvas
    extends ChessCanvas
        implements MessageListener, SelectListener
{
    private static final String LOVER_ID = "sample";

    // messageReturned
    private static final int TAG_CLEAR = 1;
    private static final int TAG_HELP  = 2;

    // selectionDone
    private static final int TAG_MENU   = 1;

    private static final int ID_ACCEPT_SETTING = 1;
    private static final int ID_CLEAR_BOARD    = 2;
    private static final int ID_CANCEL = 3;
    private static final int ID_HELP   = 4;

    private int px, py;
    private int mainIndex = 0;

    private Localizer localizer;
    private static PositionCanvas instance;

    private Board b;
    private int color;

    private int piece;

    /**
     * Creates a new instance of PositionCanvas
     */
    PositionCanvas (MIDlet midlet, Localizer localizer)
    {
        super (midlet);
        this.localizer = localizer;
        init();
    }

    static PositionCanvas GetInstance (MIDlet midlet, Localizer localizer)
    {
        return
            instance = new PositionCanvas (midlet, localizer);
    }

    private void init()
    {
    }

    public void keyPressed (int code)
    {
        cursorCount = 0;
        boolean done = true;

        switch (code)
        {
            default:
                done = false;
                break;

            case KEY_STAR:
            case KEY_POUND:
                break;

            case KEY_NUM0: setPiece (EMPTY);  break;
            case KEY_NUM1: setPiece (PAWN);   break;
            case KEY_NUM2: setPiece (KNIGHT); break;
            case KEY_NUM3: setPiece (BISHOP); break;
            case KEY_NUM4: setPiece (ROOK);   break;
            case KEY_NUM5: setPiece (QUEEN);  break;
            case KEY_NUM6: setPiece (KING);   break;

            case KEY_NUM7:
                ChessCanvas.BacklightOn ^= true;
                break;

            case KEY_NUM8:
                showLetters = !showLetters;
                break;

            case KEY_NUM9:
                mainIndex = 0;
                showMainMenu ();
                return;
        }

        if (!done)
            handleSample (getGameAction (code));

        save (LOVER_ID);
        repaint();
    }

    private void showMainMenu ()
    {
        Selector.Show (this, TAG_MENU, L._("T_TITLE"), new SelectorLabel[]
        {
            new SelectorLabel (ID_ACCEPT_SETTING,  L._("T_ACCEPT_POSITION")),
            new SelectorLabel (ID_CLEAR_BOARD,  L._("T_CLEAR_BOARD")),
            new SelectorLabel (ID_HELP,    L._("T_HELP")),
            new SelectorLabel (ID_CANCEL,  L._("T_CANCEL"))
        });
        
        if (mainIndex >=0)
            Selector.SetSelectedIndex (mainIndex);
    }

    private int nClicks = 0;

    private void setPiece (int p)
    {

        if (piece != p)
        {
            piece = p;
            nClicks = 0;
            return;
        }

        if (EMPTY == p)
        {
            if (++nClicks % 2 == 1)
            {
                b.mover = 1-b.mover;
                b.save (LOVER_ID);
            }
            else
                turn = !turn;
        }
        else
            color = 1-color;
    }

    private void handleSample (int game)
    {
        boolean done = true;

        int dd = turn ? -1 : 1;

        switch (game)
        {
            default:               return;
            case FIRE:             done = false;            break;
            case UP:               py += dd;                break;
            case DOWN:             py -= dd;                break;
            case RIGHT:            px += dd;                break;
            case LEFT:             px -= dd;                break;
        }

        if (done)
        {
            px = Math.min (7, Math.max(0, px));
            py = Math.min (7, Math.max(0, py));
            nClicks = 0;
            return;
        }

        // FIRE

        Field f = Field.Get (px, py);

        if (KING == piece)
        {
            Field king = b.king[color];

            if (f.equals (b.king[1-color]))
                b.king[1-color] = null;

            if (null != king)
                b.p[king.x][king.y] = EMPTY;

            b.king[color] = f;
        }
        else
        {
            if (PAWN == piece  &&  (py == 0 || py == 7))
                return;
            if (f.equals (b.king[WHITE]))     b.king[WHITE] = null;
            if (f.equals (b.king[BLACK]))     b.king[BLACK] = null;
        }

        b.p[px][py] = piece | ((piece == EMPTY) ? 0 : color);

        b.startSaved = false;
        b.save (LOVER_ID);
    }

    public void paint (Graphics g)
    {
        setDrawContext (g, b, b.mover);

        g.translate(- g.getTranslateX(), - g.getTranslateY());

        drawBackground (0x600000);

        g.translate (getWidth()/2-4*SIZE, getHeight()/2-4*SIZE);

        drawFrame ();
        //Drawer.setDrawContext (g, b, color);
        drawBoard ();
        drawPieces ();

        if (b.isCheck (WHITE))            drawCheck (WHITE);
        if (b.isCheck (BLACK))            drawCheck (BLACK);

        drawCursor (-(piece|color), px, py);
    }


    static void Show (boolean newSample)
    {
        if (newSample)
            instance.startSample();
        else
            instance.resume();
    }

    private void startSample ()
    {
        b = new Board();

        if (!b.load (LOVER_ID))
            b.clear();

        load (LOVER_ID, b.mover);
        piece = KING;
        color = WHITE;

        px = 4;
        py = 0;

        resume();
    }

    private void resume ()
    {
        //Drawer.canvas = this;

        setFullScreenMode (true);
        Display.getDisplay (midlet).setCurrent (this);
        repaint();
    }

    public void selectionDone (int id, int tag, int index)
    {
        mainIndex = index;
        
        if (TAG_MENU == tag)
        {
            mainIndex = index;
            switch (id)
            {
                case Selector.BACK:
                    resume();
                    return;

                case ID_ACCEPT_SETTING:
                    //ChessLET.debug ("Accept");
                    PositionInfoForm.ProcessBoard (b);
                    return;

                case ID_CLEAR_BOARD:
                    //ChessLET.debug ("Clear");
                    //Display.getDisplay (midlet).setCurrent (this);
                    Message.Show (this, TAG_CLEAR, Message.CONFIRM, L._("T_ASK_CLEAR_BOARD"));
                    return;

                case ID_CANCEL:
                    PlayCanvas.Show (false);
                    return;
                    
                case ID_HELP:
                    TextForm.Show (this, TAG_HELP, L._("T_TITLE"), getHelpItems());
                    return;
                    
            }
        }
    }
    
    private String[][] getHelpItems ()
    {
        return new String[][]
        {
            {L._("L_UDLR"), L._("POS.UDLR")}, 
            {L._("L_FIRE"), L._("POS.FIRE")}, 
            {"0",   L._("POS.0")}, 
            {"1-6", L._("POS.1-6")}, 
            {"1",   L._("POS.1")}, 
            {"2",   L._("POS.2")}, 
            {"3",   L._("POS.3")}, 
            {"4",   L._("POS.4")}, 
            {"5",   L._("POS.5")}, 
            {"6",   L._("POS.6")}, 
            {"7",   L._("POS.7")}, 
            {"8",   L._("POS.8")}, 
            {"9",   L._("POS.9")} 
        };
    }
    

    public void messageReturned (int id, int tag)
    {
        if (TAG_HELP == tag)
        {
            showMainMenu ();
        }
        
        if (TAG_CLEAR == tag)
        {
            switch (id)
            {
                case Command.OK:
                    startSample();
                    b.clear();
                    break;

                case Command.CANCEL:
                    showMainMenu ();
                    break;
            }
        }
    }
}

